#ifndef CLUT_GEN_H_
#define CLUT_GEN_H_

//
//   Clut generators
//


namespace clut_miser
{
 //
 // shade coefs used in generating cluts
 //
 struct output_settings
 {
   int default_shade_bits; // newly created palettes use this value
                           // to determine the number of bits to use
                           // in making a clut
                           // The number of distinct shades in a color table is 2^shade_bits.
                           // Higher values will result in less banding
                           // Lower values will use less memory and thus result in superior performance
                           // Recommended range:	4 - 6.
                           // Default setting:		6 

   float diffuse_scale ; // This is the scale applied to a source r,g or b 
                         // component when diffuse lighting is at a maximum.
                        // Recommended range:	1.0 .. 2.0
                        // Default setting: 		1.25 

   float specular_start; // This is the threshold at which specular lighting is applied to the output.
                         // This value must be coordinated with the current 3d lighting system.
                         //  Recommended range: 	0.5 .. 1.0.
                         //  Default setting: 		0.75 
   float specular_scale; // This is the intensity of white specular light that is
                         // applied to a pixel when specular lighting is at a maximum.
                         //  Recommended range: 	0.0 .. 1.0
                         //  Default setting: 		1.0 
                         //  A value of 0 will apply no specular component. (e.g. a matte surface.)
                         //  A value of 1.0 will produce a shiny material.  (in appropriate 3d lighting conditions)
   
   output_settings();
 };
 


 //
 // A structure used to create cluts
 //
 struct generator
 {
 
  
  //
  // returns the number of bytes that a clut created for 'table'
  // would occupy.
  //
  
  int calculate_clut_size(const color_table & table, int shade_bits = -1) const;

  //
  // creates a clut based on color_table 'tbl' and
  // stores the result in 'clut'.
  //

  //
  // - clut is allocated by caller.
  //
  // - the number of bytes written to clut is 
  //   the value returned by calculate_clut_size, 
  //
  // - for best performance, should clut be aligned on 
  //   a 32 byte boundary 
  //
  void make_clut( clut_t clut,
                  const color_table & table,
                          int shade_bits = -1 ) const;

   
  // returns a good alignment on which to allocate
  // clut memory
  u32 suggest_clut_alignment(int shade_bits = -1) const;
  

  
  
  output_settings settings;  // current settings
                             // see output settings


  // number of bytes used per pixel on the generators target display type.
  // this is 2 for 16 bit, 1 for 8 bit.
  const int bytes_per_pixel;
  

 //
 // r,g,b in 0..255 range
 // duplicates pixel 4/bytes_per_pixel times;
 // eg in 16 bit returns color | (color<<16);
 // used internally
  virtual u32 color_match( u32 r, u32 g, u32 b ) const  = 0;
 
  generator( int _bytes_per_pixel);
 };

 //
 // Generator used to generate cluts for a 16 bit rgb display.
 //
 struct generator_16 : generator
 {
  generator_16( const pixel_format_16 & format );

  pixel_format_16 format;

  virtual u32 color_match( u32 r, u32 g, u32 b ) const;
 };


 //
 // Generator used to generate cluts for an 8 bit palletized display.
 //
 struct generator_8 : generator
 {
  

  generator_8(const color_table & _video_palette);

  // 
  // Generates a pixel blend table for use by raster.
  // This function takes a considerable amount of time to execute.
  //  The table generated is 64k.  
  //  It can be created once for a given video palette and saved.
  
  pixel_blend_table_8 create_blend_table() const;
  


 
 
  color_table video_palette;
  
  // this is an expensive funtion call.
  // it does an exhaustive search to find the 
  // best match based on an l2 norm metric.
  virtual u32 color_match( u32 r, u32 g, u32 b ) const;

 };

}

#endif